@@ -52,6 +52,7 @@ struct _BusEngineProxy {
        IBusInputContextPrivate) */
     IBusText *surrounding_text;
     guint     surrounding_cursor_pos;
+    guint     selection_anchor_pos;
 };
 
 struct _BusEngineProxyClass {
@@ -357,7 +358,6 @@ static void
 bus_engine_proxy_init (BusEngineProxy *engine)
 {
     engine->surrounding_text = g_object_ref_sink (text_empty);
-    engine->surrounding_cursor_pos = 0;
 }
 
 static void
@@ -397,15 +397,6 @@ bus_engine_proxy_real_destroy (IBusProxy *proxy)
 {
     BusEngineProxy *engine = (BusEngineProxy *)proxy;
 
-    g_dbus_proxy_call ((GDBusProxy *)proxy,
-                       "org.freedesktop.IBus.Service.Destroy",
-                       NULL,
-                       G_DBUS_CALL_FLAGS_NONE,
-                       -1,
-                       NULL,
-                       NULL,
-                       NULL);
-
     if (engine->desc) {
         g_object_unref (engine->desc);
         engine->desc = NULL;
@@ -603,7 +594,8 @@ bus_engine_proxy_new_internal (const gchar     *path,
     g_assert (IBUS_IS_ENGINE_DESC (desc));
     g_assert (G_IS_DBUS_CONNECTION (connection));
 
-
+    GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
+                            G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES;
     BusEngineProxy *engine =
         (BusEngineProxy *) g_initable_new (BUS_TYPE_ENGINE_PROXY,
                                            NULL,
@@ -613,7 +605,7 @@ bus_engine_proxy_new_internal (const gchar     *path,
                                            "g-interface-name",  IBUS_INTERFACE_ENGINE,
                                            "g-object-path",     path,
                                            "g-default-timeout", g_gdbus_timeout,
-                                           "g-flags",           G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START | G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
+                                           "g-flags",           flags,
                                            NULL);
     const gchar *layout = ibus_engine_desc_get_layout (desc);
     if (layout != NULL && layout[0] != '\0') {
@@ -638,24 +630,33 @@ static void
 engine_proxy_new_data_free (EngineProxyNewData *data)
 {
     if (data->simple != NULL) {
-        if (data->handler_id != 0)
-            g_signal_handler_disconnect (data->component, data->handler_id);
         g_object_unref (data->simple);
     }
 
-    if (data->component != NULL)
+    if (data->desc != NULL) {
+        g_object_unref (data->desc);
+    }
+
+    if (data->component != NULL) {
+        if (data->handler_id != 0) {
+            g_signal_handler_disconnect (data->component, data->handler_id);
+        }
         g_object_unref (data->component);
+    }
 
-    if (data->factory != NULL)
+    if (data->factory != NULL) {
         g_object_unref (data->factory);
+    }
 
-    if (data->timeout_id != 0)
+    if (data->timeout_id != 0) {
         g_source_remove (data->timeout_id);
+    }
 
     if (data->cancellable != NULL) {
-        if (data->cancelled_handler_id != 0)
+        if (data->cancelled_handler_id != 0) {
             g_cancellable_disconnect (data->cancellable,
-                data->cancelled_handler_id);
+                                      data->cancelled_handler_id);
+        }
         g_object_unref (data->cancellable);
     }
 
@@ -772,7 +773,8 @@ timeout_cb (EngineProxyNewData *data)
 /**
  * cancelled_cb:
  *
- * A callback function to be called when someone calls g_cancellable_cancel() for the cancellable object for bus_engine_proxy_new.
+ * A callback function to be called when someone calls g_cancellable_cancel()
+ * for the cancellable object for bus_engine_proxy_new.
  * Call the GAsyncReadyCallback.
  */
 static gboolean
@@ -793,8 +795,12 @@ static void
 cancelled_cb (GCancellable       *cancellable,
               EngineProxyNewData *data)
 {
-    /* Cancel the bus_engine_proxy_new() in idle to avoid deadlock */
-    g_idle_add ((GSourceFunc) cancelled_idle_cb, data);
+    /* Cancel the bus_engine_proxy_new() in idle to avoid deadlock.
+     * And use HIGH priority to avoid timeout event happening before
+     * idle callback. */
+    g_idle_add_full (G_PRIORITY_HIGH,
+                    (GSourceFunc) cancelled_idle_cb,
+                    data, NULL);
 }
 
 void
@@ -831,13 +837,12 @@ bus_engine_proxy_new (IBusEngineDesc      *desc,
     data->simple = simple;
     data->timeout = timeout;
 
-    g_object_set_data ((GObject *)data->simple, "EngineProxyNewData", data);
-
     data->factory = bus_component_get_factory (data->component);
 
     if (data->factory == NULL) {
-        /* The factory is not ready yet. Create the factory first, and wait for the "notify::factory" signal.
-         * In the handler of "notify::factory", we'll create the engine proxy. */
+        /* The factory is not ready yet. Create the factory first, and wait for
+         * the "notify::factory" signal. In the handler of "notify::factory",
+         * we'll create the engine proxy. */
         data->handler_id = g_signal_connect (data->component,
                                              "notify::factory",
                                              G_CALLBACK (notify_factory_cb),
@@ -902,7 +907,7 @@ bus_engine_proxy_process_key_event (BusEngineProxy      *engine,
         if (keymap == NULL)
             keymap = BUS_DEFAULT_KEYMAP;
         if (keymap != NULL) {
-            guint t = ibus_keymap_lookup_keysym (engine->keymap, keycode, state);
+            guint t = ibus_keymap_lookup_keysym (keymap, keycode, state);
             if (t != IBUS_VoidSymbol) {
                 keyval = t;
             }
@@ -944,6 +949,40 @@ bus_engine_proxy_set_cursor_location (BusEngineProxy *engine,
     }
 }
 
+void
+bus_engine_proxy_process_hand_writing_event
+                                  (BusEngineProxy        *engine,
+                                   GVariant              *coordinates)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+
+    g_dbus_proxy_call ((GDBusProxy *)engine,
+                       "ProcessHandWritingEvent",
+                       coordinates,
+                       G_DBUS_CALL_FLAGS_NONE,
+                       -1,
+                       NULL,
+                       NULL,
+                       NULL);
+}
+
+void
+bus_engine_proxy_cancel_hand_writing
+                                  (BusEngineProxy        *engine,
+                                   guint                  n_strokes)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+
+    g_dbus_proxy_call ((GDBusProxy *)engine,
+                       "CancelHandWriting",
+                       g_variant_new ("(u)", n_strokes),
+                       G_DBUS_CALL_FLAGS_NONE,
+                       -1,
+                       NULL,
+                       NULL,
+                       NULL);
+}
+
 void
 bus_engine_proxy_set_capabilities (BusEngineProxy *engine,
                                    guint           caps)
@@ -1016,23 +1055,29 @@ void bus_engine_proxy_property_hide (BusEngineProxy *engine,
 
 void bus_engine_proxy_set_surrounding_text (BusEngineProxy *engine,
                                             IBusText       *text,
-                                            guint           cursor_pos)
+                                            guint           cursor_pos,
+                                            guint           anchor_pos)
 {
     g_assert (BUS_IS_ENGINE_PROXY (engine));
     g_assert (text != NULL);
 
     if (!engine->surrounding_text ||
         g_strcmp0 (text->text, engine->surrounding_text->text) != 0 ||
-        cursor_pos != engine->surrounding_cursor_pos) {
+        cursor_pos != engine->surrounding_cursor_pos ||
+        anchor_pos != engine->selection_anchor_pos) {
         GVariant *variant = ibus_serializable_serialize ((IBusSerializable *)text);
         if (engine->surrounding_text)
             g_object_unref (engine->surrounding_text);
         engine->surrounding_text = (IBusText *) g_object_ref_sink (text);
         engine->surrounding_cursor_pos = cursor_pos;
+        engine->selection_anchor_pos = anchor_pos;
 
         g_dbus_proxy_call ((GDBusProxy *)engine,
                            "SetSurroundingText",
-                           g_variant_new ("(vu)", variant, cursor_pos),
+                           g_variant_new ("(vuu)",
+                                          variant,
+                                          cursor_pos,
+                                          anchor_pos),
                            G_DBUS_CALL_FLAGS_NONE,
                            -1,
                            NULL,
