@@ -238,6 +238,12 @@ static const gchar introspection_xml[] =
     "      <arg direction='in' type='i' name='w' />"
     "      <arg direction='in' type='i' name='h' />"
     "    </method>"
+    "    <method name='ProcessHandWritingEvent'>"
+    "      <arg direction='in' type='ad' name='coordinates' />"
+    "    </method>"
+    "    <method name='CancelHandWriting'>"
+    "      <arg direction='in' type='u' name='n_strokes' />"
+    "    </method>"
     "    <method name='FocusIn' />"
     "    <method name='FocusOut' />"
     "    <method name='Reset' />"
@@ -262,6 +268,7 @@ static const gchar introspection_xml[] =
     "    <method name='SetSurroundingText'>"
     "      <arg direction='in' type='v' name='text' />"
     "      <arg direction='in' type='u' name='cursor_pos' />"
+    "      <arg direction='in' type='u' name='anchor_pos' />"
     "    </method>"
 
     /* signals */
@@ -779,6 +786,35 @@ _ic_set_cursor_location (BusInputContext       *context,
     }
 }
 
+static void
+_ic_process_hand_writing_event (BusInputContext       *context,
+                                GVariant              *parameters,
+                                GDBusMethodInvocation *invocation)
+{
+    /* do nothing if it is a fake input context */
+    if (context->has_focus && context->enabled &&
+        context->engine && context->fake == FALSE) {
+        bus_engine_proxy_process_hand_writing_event (context->engine, parameters);
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+}
+
+static void
+_ic_cancel_hand_writing (BusInputContext       *context,
+                         GVariant              *parameters,
+                         GDBusMethodInvocation *invocation)
+{
+    guint n_strokes = 0;
+    g_variant_get (parameters, "(u)", &n_strokes);
+
+    /* do nothing if it is a fake input context */
+    if (context->has_focus && context->enabled &&
+        context->engine && context->fake == FALSE) {
+        bus_engine_proxy_cancel_hand_writing (context->engine, n_strokes);
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
+}
+
 /**
  * _ic_focus_in:
  *
@@ -985,8 +1021,6 @@ _ic_set_engine (BusInputContext       *context,
                             NULL,
                             (GAsyncReadyCallback)_ic_set_engine_done,
                             invocation);
-
-    g_object_unref (desc);
 }
 
 /**
@@ -1005,8 +1039,11 @@ _ic_get_engine (BusInputContext       *context,
                         g_variant_new ("(v)", ibus_serializable_serialize ((IBusSerializable *)desc)));
     }
     else {
-        g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
-                        "Input context does not have engine.");
+        g_dbus_method_invocation_return_error (
+                invocation,
+                IBUS_ERROR,
+                IBUS_ERROR_NO_ENGINE,
+                "Input context does not have engine.");
     }
 }
 
@@ -1023,8 +1060,13 @@ _ic_set_surrounding_text (BusInputContext       *context,
     GVariant *variant = NULL;
     IBusText *text;
     guint cursor_pos = 0;
+    guint anchor_pos = 0;
 
-    g_variant_get (parameters, "(vu)", &variant, &cursor_pos);
+    g_variant_get (parameters,
+                   "(vuu)",
+                   &variant,
+                   &cursor_pos,
+                   &anchor_pos);
     text = IBUS_TEXT (ibus_serializable_deserialize (variant));
     g_variant_unref (variant);
 
@@ -1032,7 +1074,8 @@ _ic_set_surrounding_text (BusInputContext       *context,
          context->has_focus && context->enabled && context->engine) {
         bus_engine_proxy_set_surrounding_text (context->engine,
                                                text,
-                                               cursor_pos);
+                                               cursor_pos,
+                                               anchor_pos);
     }
 
     if (g_object_is_floating (text))
@@ -1070,6 +1113,9 @@ bus_input_context_service_method_call (IBusService            *service,
     } methods [] =  {
         { "ProcessKeyEvent",   _ic_process_key_event },
         { "SetCursorLocation", _ic_set_cursor_location },
+        { "ProcessHandWritingEvent",
+                               _ic_process_hand_writing_event },
+        { "CancelHandWriting", _ic_cancel_hand_writing },
         { "FocusIn",           _ic_focus_in },
         { "FocusOut",          _ic_focus_out },
         { "Reset",             _ic_reset },
@@ -1132,7 +1178,6 @@ bus_input_context_focus_in (BusInputContext *context)
                             NULL, /* we do not cancel the call. */
                             NULL, /* use the default callback function. */
                             NULL);
-            g_object_unref (desc);
         }
     }
 
@@ -2050,7 +2095,6 @@ bus_input_context_enable (BusInputContext *context)
                             NULL, /* we do not cancel the call. */
                             NULL, /* use the default callback function. */
                             NULL);
-            g_object_unref (desc);
         }
     }
 
@@ -2151,7 +2195,6 @@ bus_input_context_unset_engine (BusInputContext *context)
         for (i = 0; engine_signals[i].name != NULL; i++) {
             g_signal_handlers_disconnect_by_func (context->engine, engine_signals[i].callback, context);
         }
-        /* Do not destroy the engine anymore, because of global engine feature */
         g_object_unref (context->engine);
         context->engine = NULL;
     }
@@ -2249,7 +2292,10 @@ new_engine_cb (GObject             *obj,
                                              "Opertation was cancelled");
         }
         else {
+            /* Let BusEngineProxy call a Disable signal. */
+            bus_input_context_disable (data->context);
             bus_input_context_set_engine (data->context, engine);
+            g_object_unref (engine);
             bus_input_context_enable (data->context);
             g_simple_async_result_set_op_res_gboolean (data->simple, TRUE);
         }
