@@ -20,7 +20,6 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <config.h>
-#include <dbus/dbus.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -29,6 +28,9 @@
 #include <stdlib.h>
 #include <locale.h>
 #include <signal.h>
+#include <glib.h>
+#include <gio/gio.h>
+#include <ibus.h>
 #include "server.h"
 #include "ibusimpl.h"
 
@@ -42,11 +44,11 @@ static gboolean restart = FALSE;
 static gchar *panel = "default";
 static gchar *config = "default";
 static gchar *desktop = "gnome";
-static gchar *address = "";
+gchar *g_address = "unix:tmpdir=/tmp";
 gchar *g_cache = "auto";
 gboolean g_mempro = FALSE;
 gboolean g_verbose = FALSE;
-gint   g_dbus_timeout = 5000;
+gint   g_gdbus_timeout = 5000;
 #ifdef G_THREADS_ENABLED
 gint   g_monitor_timeout = 0;
 #endif
@@ -65,12 +67,12 @@ static const GOptionEntry entries[] =
     { "single",    's', 0, G_OPTION_ARG_NONE,   &single,    "do not execute panel and config module.", NULL },
     { "xim",       'x', 0, G_OPTION_ARG_NONE,   &xim,       "execute ibus XIM server.", NULL },
     { "desktop",   'n', 0, G_OPTION_ARG_STRING, &desktop,   "specify the name of desktop session. [default=gnome]", "name" },
-    { "panel",     'p', 0, G_OPTION_ARG_STRING, &panel,     "specify the cmdline of panel program.", "cmdline" },
-    { "config",    'c', 0, G_OPTION_ARG_STRING, &config,    "specify the cmdline of config program.", "cmdline" },
-    { "address",   'a', 0, G_OPTION_ARG_STRING, &address,   "specify the address of ibus daemon.", "address" },
+    { "panel",     'p', 0, G_OPTION_ARG_STRING, &panel,     "specify the cmdline of panel program. pass 'disable' not to start a panel program.", "cmdline" },
+    { "config",    'c', 0, G_OPTION_ARG_STRING, &config,    "specify the cmdline of config program. pass 'disable' not to start a config program.", "cmdline" },
+    { "address",   'a', 0, G_OPTION_ARG_STRING, &g_address,   "specify the address of ibus daemon.", "address" },
     { "replace",   'r', 0, G_OPTION_ARG_NONE,   &replace,   "if there is an old ibus-daemon is running, it will be replaced.", NULL },
     { "cache",     't', 0, G_OPTION_ARG_STRING, &g_cache,   "specify the cache mode. [auto/refresh/none]", NULL },
-    { "timeout",   'o', 0, G_OPTION_ARG_INT,    &g_dbus_timeout, "dbus reply timeout in milliseconds.", "timeout [default is 2000]" },
+    { "timeout",   'o', 0, G_OPTION_ARG_INT,    &g_gdbus_timeout, "gdbus reply timeout in milliseconds. pass -1 to use the default timeout of gdbus.", "timeout [default is 5000]" },
 #ifdef G_THREADS_ENABLED
     { "monitor-timeout", 'j', 0, G_OPTION_ARG_INT,    &g_monitor_timeout, "timeout of poll changes of engines in seconds. 0 to disable it. ", "timeout [default is 0]" },
 #endif
@@ -80,17 +82,22 @@ static const GOptionEntry entries[] =
     { NULL },
 };
 
+/**
+ * execute_cmdline:
+ * @cmdline: An absolute path of the executable and its parameters, e.g.  "/usr/lib/ibus/ibus-x11 --kill-daemon".
+ * @returns: TRUE if both parsing cmdline and executing the command succeed.
+ *
+ * Execute cmdline. Child process's stdin, stdout, and stderr are attached to /dev/null.
+ * You don't have to handle SIGCHLD from the child process since glib will do.
+ */
 static gboolean
 execute_cmdline (const gchar *cmdline)
 {
     g_assert (cmdline);
 
-    gint argc;
-    gchar **argv;
-    gboolean retval;
-    GError *error;
-
-    error = NULL;
+    gint argc = 0;
+    gchar **argv = NULL;
+    GError *error = NULL;
     if (!g_shell_parse_argv (cmdline, &argc, &argv, &error)) {
         g_warning ("Can not parse cmdline `%s` exec: %s", cmdline, error->message);
         g_error_free (error);
@@ -98,7 +105,7 @@ execute_cmdline (const gchar *cmdline)
     }
 
     error = NULL;
-    retval = g_spawn_async (NULL, argv, NULL,
+    gboolean retval = g_spawn_async (NULL, argv, NULL,
                             G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
                             NULL, NULL,
                             NULL, &error);
@@ -156,6 +163,12 @@ daemon (gint nochdir, gint noclose)
 }
 #endif
 
+/*
+ * _sig_usr2_handler:
+ * @sig: the signal number, which is usually SIGUSR2.
+ *
+ * A signal handler for SIGUSR2 signal. Dump a summary of memory usage to stderr.
+ */
 static void
 _sig_usr2_handler (int sig)
 {
@@ -165,21 +178,20 @@ _sig_usr2_handler (int sig)
 gint
 main (gint argc, gchar **argv)
 {
-    GOptionContext *context;
-    BusServer *server;
-    IBusBus *bus;
-
-    GError *error = NULL;
-
     setlocale (LC_ALL, "");
 
-    context = g_option_context_new ("- ibus daemon");
-
+    GOptionContext *context = g_option_context_new ("- ibus daemon");
     g_option_context_add_main_entries (context, entries, "ibus-daemon");
 
     g_argv = g_strdupv (argv);
+    GError *error = NULL;
     if (!g_option_context_parse (context, &argc, &argv, &error)) {
         g_printerr ("Option parsing failed: %s\n", error->message);
+	g_error_free (error);
+        exit (-1);
+    }
+    if (g_gdbus_timeout < -1) {
+        g_printerr ("Bad timeout (must be >= -1): %d\n", g_gdbus_timeout);
         exit (-1);
     }
 
@@ -208,19 +220,19 @@ main (gint argc, gchar **argv)
         }
     }
 
-    /* create a new process group */
+    /* create a new process group. this is important to kill all of its children by SIGTERM at a time in bus_ibus_impl_destroy. */
     setpgid (0, 0);
 
-    g_type_init ();
+    ibus_init ();
 
 #ifdef G_THREADS_ENABLED
     g_thread_init (NULL);
 #endif
-    ibus_set_log_handler(g_verbose);
+    ibus_set_log_handler (g_verbose);
 
     /* check if ibus-daemon is running in this session */
     if (ibus_get_address () != NULL) {
-        bus = ibus_bus_new ();
+        IBusBus *bus = ibus_bus_new ();
 
         if (ibus_bus_is_connected (bus)) {
             if (!replace) {
@@ -233,22 +245,18 @@ main (gint argc, gchar **argv)
             }
         }
         g_object_unref (bus);
-        bus = NULL;
     }
 
-    /* create ibus server */
-    server = bus_server_get_default ();
-    bus_server_listen (server);
-
+    bus_server_init ();
     if (!single) {
         /* execute config component */
         if (g_strcmp0 (config, "default") == 0) {
-            IBusComponent *component;
+            BusComponent *component;
             component = bus_registry_lookup_component_by_name (BUS_DEFAULT_REGISTRY, IBUS_SERVICE_CONFIG);
             if (component) {
-                ibus_component_set_restart (component, restart);
+                bus_component_set_restart (component, restart);
             }
-            if (component == NULL || !ibus_component_start (component, g_verbose)) {
+            if (component == NULL || !bus_component_start (component, g_verbose)) {
                 g_printerr ("Can not execute default config program\n");
                 exit (-1);
             }
@@ -257,14 +265,14 @@ main (gint argc, gchar **argv)
                 exit (-1);
         }
 
-        /* execut panel component */
+        /* execute panel component */
         if (g_strcmp0 (panel, "default") == 0) {
-            IBusComponent *component;
+            BusComponent *component;
             component = bus_registry_lookup_component_by_name (BUS_DEFAULT_REGISTRY, IBUS_SERVICE_PANEL);
             if (component) {
-                ibus_component_set_restart (component, restart);
+                bus_component_set_restart (component, restart);
             }
-            if (component == NULL || !ibus_component_start (component, g_verbose)) {
+            if (component == NULL || !bus_component_start (component, g_verbose)) {
                 g_printerr ("Can not execute default panel program\n");
                 exit (-1);
             }
@@ -276,11 +284,10 @@ main (gint argc, gchar **argv)
 
     /* execute ibus xim server */
     if (xim) {
-        if (!execute_cmdline (LIBEXECDIR"/ibus-x11 --kill-daemon"))
+        if (!execute_cmdline (LIBEXECDIR "/ibus-x11 --kill-daemon"))
             exit (-1);
     }
 
-    bus_server_run (server);
-
+    bus_server_run ();
     return 0;
 }
