@@ -94,6 +94,11 @@ class Component(Serializable):
         engine = EngineDesc(name, longname, description, language, license, author, icon, layout, hotkeys)
         self.__engines.append(engine)
 
+    def add_engines(self, engines):
+        if not isinstance(engines, list):
+            raise TypeError("engines must be an instance of list")
+        self.__engines.extend(engines)
+
     def serialize(self, struct):
         super(Component, self).serialize(struct)
         struct.append (dbus.String(self.__name))
@@ -106,8 +111,6 @@ class Component(Serializable):
         struct.append (dbus.String(self.__textdomain))
         struct.append (dbus.Array(map(serialize_object,self.__observed_paths), signature="v"))
         struct.append (dbus.Array(map(serialize_object,self.__engines), signature="v"))
-        # New properties of Component will use dict for serialize
-        struct.append(dbus.Array({}, signature=None))
 
     def deserialize(self, struct):
         super(Component, self).deserialize(struct)
@@ -123,8 +126,6 @@ class Component(Serializable):
 
         self.__observed_paths = map(deserialize_object, struct.pop(0))
         self.__engines = map(deserialize_object, struct.pop(0))
-        # New properties of Component will use dict for serialize
-        #value = struct.pop(0)
 
 def test():
     text = Component("Hello", "", "", "", "", "", "", "")
