@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2009 Red Hat, Inc.
+# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -185,7 +185,7 @@ class PropList(Serializable):
     def append(self, prop):
         self.__props.append(prop)
 
-    def prepand(self, prop):
+    def prepend(self, prop):
         self.__props.insert(0, prop)
 
     def insert(self, index, prop):
