@@ -19,33 +19,30 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <dbus/dbus.h>
 #include "ibusattrlist.h"
 
 /* functions prototype */
 static void         ibus_attr_list_destroy      (IBusAttrList           *attr_list);
 static gboolean     ibus_attr_list_serialize    (IBusAttrList           *attr_list,
-                                                 IBusMessageIter        *iter);
-static gboolean     ibus_attr_list_deserialize  (IBusAttrList           *attr_list,
-                                                 IBusMessageIter        *iter);
+                                                 GVariantBuilder        *builder);
+static gint         ibus_attr_list_deserialize  (IBusAttrList           *attr_list,
+                                                 GVariant               *variant);
 static gboolean     ibus_attr_list_copy         (IBusAttrList           *dest,
                                                  const IBusAttrList     *src);
 
 G_DEFINE_TYPE (IBusAttrList, ibus_attr_list, IBUS_TYPE_SERIALIZABLE)
 
 static void
-ibus_attr_list_class_init (IBusAttrListClass *klass)
+ibus_attr_list_class_init (IBusAttrListClass *class)
 {
-    IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
-    IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (klass);
+    IBusObjectClass *object_class = IBUS_OBJECT_CLASS (class);
+    IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (class);
 
     object_class->destroy = (IBusObjectDestroyFunc) ibus_attr_list_destroy;
 
     serializable_class->serialize   = (IBusSerializableSerializeFunc) ibus_attr_list_serialize;
     serializable_class->deserialize = (IBusSerializableDeserializeFunc) ibus_attr_list_deserialize;
     serializable_class->copy        = (IBusSerializableCopyFunc) ibus_attr_list_copy;
-
-    g_string_append (serializable_class->signature, "av");
 }
 
 static void
@@ -77,66 +74,47 @@ ibus_attr_list_destroy (IBusAttrList *attr_list)
 
 static gboolean
 ibus_attr_list_serialize (IBusAttrList    *attr_list,
-                          IBusMessageIter *iter)
+                          GVariantBuilder *builder)
 {
-    IBusMessageIter array_iter;
     gboolean retval;
     guint i;
 
-    retval = IBUS_SERIALIZABLE_CLASS (ibus_attr_list_parent_class)->serialize ((IBusSerializable *)attr_list, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_attr_list_parent_class)->serialize ((IBusSerializable *)attr_list, builder);
     g_return_val_if_fail (retval, FALSE);
 
     g_return_val_if_fail (IBUS_IS_ATTR_LIST (attr_list), FALSE);
 
-    retval = ibus_message_iter_open_container (iter,
-                                               IBUS_TYPE_ARRAY,
-                                               "v",
-                                               &array_iter);
-    g_return_val_if_fail (retval, FALSE);
+    GVariantBuilder array;
+    g_variant_builder_init (&array, G_VARIANT_TYPE ("av"));
 
     for (i = 0;; i++) {
         IBusAttribute *attr;
-
         attr = ibus_attr_list_get (attr_list, i);
         if (attr == NULL)
             break;
-
-        retval = ibus_message_iter_append (&array_iter, IBUS_TYPE_ATTRIBUTE, &attr);
-        g_return_val_if_fail (retval, FALSE);
+        g_variant_builder_add (&array, "v", ibus_serializable_serialize ((IBusSerializable *)attr));
     }
-
-    retval = ibus_message_iter_close_container (iter, &array_iter);
-    g_return_val_if_fail (retval, FALSE);
+    g_variant_builder_add (builder, "av", &array);
 
     return TRUE;
 }
 
-static gboolean
+static gint
 ibus_attr_list_deserialize (IBusAttrList    *attr_list,
-                            IBusMessageIter *iter)
+                            GVariant        *variant)
 {
-    DBusMessageIter array_iter;
-    gboolean retval;
+    gint retval = IBUS_SERIALIZABLE_CLASS (ibus_attr_list_parent_class)->deserialize ((IBusSerializable *)attr_list, variant);
+    g_return_val_if_fail (retval, 0);
 
-    retval = IBUS_SERIALIZABLE_CLASS (ibus_attr_list_parent_class)->deserialize ((IBusSerializable *)attr_list, iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    retval = ibus_message_iter_recurse (iter, IBUS_TYPE_ARRAY, &array_iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    while (ibus_message_iter_get_arg_type (&array_iter) != G_TYPE_INVALID) {
-        IBusAttribute *attr;
-
-        retval = ibus_message_iter_get (&array_iter, IBUS_TYPE_ATTRIBUTE, &attr);
-        g_return_val_if_fail (retval, FALSE);
-        ibus_message_iter_next (&array_iter);
-
-        ibus_attr_list_append (attr_list, attr);
+    GVariantIter *iter = NULL;
+    g_variant_get_child (variant, retval++, "av", &iter);
+    GVariant *var;
+    while (g_variant_iter_loop (iter, "v", &var)) {
+        ibus_attr_list_append (attr_list, IBUS_ATTRIBUTE (ibus_serializable_deserialize (var)));
     }
+    g_variant_iter_free (iter);
 
-    ibus_message_iter_next (iter);
-
-    return TRUE;
+    return retval;
 }
 
 
