@@ -330,7 +330,7 @@ ibus_engine_class_init (IBusEngineClass *class)
             G_TYPE_FROM_CLASS (gobject_class),
             G_SIGNAL_RUN_LAST,
             G_STRUCT_OFFSET (IBusEngineClass, process_key_event),
-            NULL, NULL,
+            g_signal_accumulator_true_handled, NULL,
             _ibus_marshal_BOOL__UINT_UINT_UINT,
             G_TYPE_BOOLEAN,
             3,
@@ -1182,17 +1182,17 @@ ibus_engine_new (const gchar     *engine_name,
                  const gchar     *object_path,
                  GDBusConnection *connection)
 {
-    return ibus_engine_new_type (IBUS_TYPE_ENGINE,
-                                 engine_name,
-                                 object_path,
-                                 connection);
+    return ibus_engine_new_with_type (IBUS_TYPE_ENGINE,
+                                      engine_name,
+                                      object_path,
+                                      connection);
 }
 
 IBusEngine  *
-ibus_engine_new_type (GType            engine_type,
-                      const gchar     *engine_name,
-                      const gchar     *object_path,
-                      GDBusConnection *connection)
+ibus_engine_new_with_type (GType            engine_type,
+                           const gchar     *engine_name,
+                           const gchar     *object_path,
+                           GDBusConnection *connection)
 {
     g_return_val_if_fail (g_type_is_a (engine_type, IBUS_TYPE_ENGINE), NULL);
     g_return_val_if_fail (engine_name != NULL, NULL);
