@@ -27,6 +27,7 @@
 #include "ibusattribute.h"
 #include "ibuslookuptable.h"
 #include "ibusproplist.h"
+#include "ibuserror.h"
 
 #define IBUS_INPUT_CONTEXT_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), IBUS_TYPE_INPUT_CONTEXT, IBusInputContextPrivate))
@@ -64,6 +65,7 @@ struct _IBusInputContextPrivate {
        BusEngineProxy) */
     IBusText *surrounding_text;
     guint     surrounding_cursor_pos;
+    guint     selection_anchor_pos;
 };
 
 typedef struct _IBusInputContextPrivate IBusInputContextPrivate;
@@ -465,7 +467,6 @@ ibus_input_context_init (IBusInputContext *context)
 
     priv = IBUS_INPUT_CONTEXT_GET_PRIVATE (context);
     priv->surrounding_text = g_object_ref_sink (text_empty);
-    priv->surrounding_cursor_pos = 0;
 }
 
 static void
@@ -715,7 +716,7 @@ ibus_input_context_new_async (const gchar         *path,
 
     GDBusProxyFlags flags = G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
                             G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES;
-    
+
     g_async_initable_new_async (IBUS_TYPE_INPUT_CONTEXT,
                                 G_PRIORITY_DEFAULT,
                                 cancellable,
@@ -804,6 +805,51 @@ ibus_input_context_get_input_context_async_finish (GAsyncResult  *res,
     return context;
 }
 
+void
+ibus_input_context_process_hand_writing_event (IBusInputContext   *context,
+                                               const gdouble      *coordinates,
+                                               guint               coordinates_len)
+{
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_return_if_fail (coordinates != NULL);
+    g_return_if_fail (coordinates_len >= 4); /* The array should contain at least one line. */
+    g_return_if_fail ((coordinates_len & 1) == 0);
+
+    guint i;
+    GVariantBuilder builder;
+    g_variant_builder_init (&builder, G_VARIANT_TYPE ("ad"));
+    for (i = 0; i < coordinates_len; i++) {
+        g_variant_builder_add (&builder, "d", coordinates[i]);
+    }
+
+    g_dbus_proxy_call ((GDBusProxy *) context,
+                       "ProcessHandWritingEvent",           /* method_name */
+                       g_variant_new ("(ad)", &builder),    /* parameters */
+                       G_DBUS_CALL_FLAGS_NONE,              /* flags */
+                       -1,                                  /* timeout */
+                       NULL,                                /* cancellable */
+                       NULL,                                /* callback */
+                       NULL                                 /* user_data */
+                       );
+}
+
+void
+ibus_input_context_cancel_hand_writing (IBusInputContext   *context,
+                                        guint               n_strokes)
+{
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+
+    g_dbus_proxy_call ((GDBusProxy *) context,
+                       "CancelHandWriting",                 /* method_name */
+                       g_variant_new ("(u)", n_strokes),    /* parameters */
+                       G_DBUS_CALL_FLAGS_NONE,              /* flags */
+                       -1,                                  /* timeout */
+                       NULL,                                /* cancellable */
+                       NULL,                                /* callback */
+                       NULL                                 /* user_data */
+                       );
+}
+
 void
 ibus_input_context_process_key_event_async (IBusInputContext   *context,
                                             guint32             keyval,
@@ -826,32 +872,27 @@ ibus_input_context_process_key_event_async (IBusInputContext   *context,
                        callback,                            /* callback */
                        user_data                            /* user_data */
                        );
-
 }
 
 gboolean
 ibus_input_context_process_key_event_async_finish (IBusInputContext  *context,
                                                    GAsyncResult      *res,
-                                                   gboolean          *processed,
                                                    GError           **error)
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
     g_assert (G_IS_ASYNC_RESULT (res));
-    g_assert (processed != NULL);
     g_assert (error == NULL || *error == NULL);
 
+    gboolean processed = FALSE;
+
     GVariant *variant = g_dbus_proxy_call_finish ((GDBusProxy *) context,
                                                    res, error);
-    if (variant == NULL) {
-        *processed = FALSE;
-        return FALSE;
-    }
-    else {
-        *processed = FALSE;
-        g_variant_get (variant, "(b)", processed);
+    if (variant != NULL) {
+        g_variant_get (variant, "(b)", &processed);
         g_variant_unref (variant);
-        return TRUE;
     }
+
+    return processed;
 }
 
 gboolean
@@ -989,28 +1030,29 @@ ibus_input_context_is_enabled_async (IBusInputContext   *context,
 gboolean
 ibus_input_context_is_enabled_async_finish (IBusInputContext   *context,
                                             GAsyncResult       *res,
-                                            gboolean           *retval,
                                             GError            **error)
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
     g_assert (G_IS_ASYNC_RESULT (res));
-    g_assert (retval != NULL);
     g_assert (error == NULL || *error == NULL);
 
+    gboolean enabled = FALSE;
+
     GVariant *variant = g_dbus_proxy_call_finish ((GDBusProxy *) context,
                                                    res, error);
-    if (variant == NULL) {
-        return FALSE;
+    if (variant != NULL) {
+        g_variant_get (variant, "(b)", &enabled);
+        g_variant_unref (variant);
     }
-    g_variant_get (variant, "(b)", retval);
-    g_variant_unref (variant);
-    return TRUE;
+
+    return enabled;
 }
 
 void
 ibus_input_context_set_surrounding_text (IBusInputContext   *context,
                                          IBusText           *text,
-                                         guint32             cursor_pos)
+                                         guint32             cursor_pos,
+                                         guint32             anchor_pos)
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
     g_assert (IBUS_IS_TEXT (text));
@@ -1018,24 +1060,31 @@ ibus_input_context_set_surrounding_text (IBusInputContext   *context,
     IBusInputContextPrivate *priv;
     priv = IBUS_INPUT_CONTEXT_GET_PRIVATE (context);
 
-    if (priv->surrounding_text == NULL ||
-        g_strcmp0 (text->text, priv->surrounding_text->text) != 0 ||
-        cursor_pos != priv->surrounding_cursor_pos) {
-        GVariant *variant = ibus_serializable_serialize ((IBusSerializable *)text);
+    if (cursor_pos != priv->surrounding_cursor_pos ||
+        anchor_pos != priv->selection_anchor_pos ||
+        priv->surrounding_text == NULL ||
+        g_strcmp0 (text->text, priv->surrounding_text->text) != 0) {
         if (priv->surrounding_text)
             g_object_unref (priv->surrounding_text);
         priv->surrounding_text = (IBusText *) g_object_ref_sink (text);
         priv->surrounding_cursor_pos = cursor_pos;
+        priv->selection_anchor_pos = anchor_pos;
 
-        g_dbus_proxy_call ((GDBusProxy *) context,
-                         "SetSurroundingText",              /* method_name */
-                         g_variant_new ("(vu)", variant, cursor_pos), /* parameters */
-                         G_DBUS_CALL_FLAGS_NONE,            /* flags */
-                         -1,                                /* timeout */
-                         NULL,                              /* cancellable */
-                         NULL,                              /* callback */
-                         NULL                               /* user_data */
-                         );
+        if (priv->needs_surrounding_text) {
+            GVariant *variant = ibus_serializable_serialize ((IBusSerializable *)text);
+            g_dbus_proxy_call ((GDBusProxy *) context,
+                               "SetSurroundingText",        /* method_name */
+                               g_variant_new ("(vuu)",
+                                              variant,
+                                              cursor_pos,
+                                              anchor_pos),  /* parameters */
+                                G_DBUS_CALL_FLAGS_NONE,     /* flags */
+                                -1,                         /* timeout */
+                                NULL,                       /* cancellable */
+                                NULL,                       /* callback */
+                                NULL                        /* user_data */
+                                );
+        }
     }
 }
 
@@ -1120,7 +1169,7 @@ IBusEngineDesc *
 ibus_input_context_get_engine (IBusInputContext *context)
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
-    GVariant *result;
+    GVariant *result = NULL;
     GError *error = NULL;
     result = g_dbus_proxy_call_sync ((GDBusProxy *) context,
                                      "GetEngine",               /* method_name */
@@ -1130,9 +1179,17 @@ ibus_input_context_get_engine (IBusInputContext *context)
                                      NULL,                      /* cancellable */
                                      &error                     /* error */
                                      );
-
     if (result == NULL) {
-        g_warning ("%s.GetEngine: %s", IBUS_INTERFACE_INPUT_CONTEXT, error->message);
+        if (g_error_matches (error, IBUS_ERROR, IBUS_ERROR_NO_ENGINE)) {
+            g_debug ("%s.GetEngine: %s",
+                     IBUS_INTERFACE_INPUT_CONTEXT,
+                     error->message);
+        }
+        else {
+            g_warning ("%s.GetEngine: %s",
+                       IBUS_INTERFACE_INPUT_CONTEXT,
+                       error->message);
+        }
         g_error_free (error);
         return NULL;
     }
@@ -1188,4 +1245,3 @@ DEFINE_FUNC(cursor_down, CursorDown);
 DEFINE_FUNC(enable, Enable);
 DEFINE_FUNC(disable, Disable);
 #undef DEFINE_FUNC
-
