@@ -124,114 +124,13 @@ ibus_serializable_destroy (IBusSerializable *serializable)
     parent_class->destroy (IBUS_OBJECT (serializable));
 }
 
-static GValue *
-ibus_g_value_dup (const GValue *value)
-{
-    GValue *new_value;
-
-    new_value = g_slice_new0 (GValue);
-    g_value_init (new_value, G_VALUE_TYPE (value));
-    g_value_copy (value, new_value);
-
-    return new_value;
-}
-
-static void
-ibus_g_value_free (GValue *value)
-{
-    g_value_unset (value);
-    g_slice_free (GValue, value);
-}
-
-static GVariant *
-_g_value_serialize (GValue          *value)
-{
-    GType type;
-
-    type = G_VALUE_TYPE (value);
-    g_return_val_if_fail (type != G_TYPE_INVALID, FALSE);
-
-    if (g_type_is_a (type, IBUS_TYPE_SERIALIZABLE)) {
-        IBusSerializable *object;
-        object = IBUS_SERIALIZABLE (g_value_get_object (value));
-        return ibus_serializable_serialize (object);
-    }
-
-    typedef const gchar *gstring;
-    switch (type) {
-#define CASE_ENTRY(TYPE, _type, signature)                              \
-    case G_TYPE_##TYPE:                                                 \
-        {                                                               \
-            g##_type v;                                                 \
-            v = g_value_get_##_type (value);                            \
-            return g_variant_new ("v", g_variant_new (signature, v));   \
-        }
-    CASE_ENTRY(CHAR, char, "y");
-    CASE_ENTRY(BOOLEAN, boolean, "b");
-    CASE_ENTRY(INT, int, "i");
-    CASE_ENTRY(UINT, uint, "u");
-    CASE_ENTRY(INT64, int64,  "x");
-    CASE_ENTRY(UINT64, uint64, "t");
-    CASE_ENTRY(FLOAT, float,  "d");
-    CASE_ENTRY(DOUBLE, double, "d");
-    CASE_ENTRY(STRING, string, "s");
-#undef CASE_ENTRY
-    }
-
-    g_assert_not_reached ();
-}
-
-static GValue *
-_g_value_deserialize (GVariant *variant)
-{
-    GValue *value = NULL;
-    const GVariantType *type;
-
-    type = g_variant_get_type (variant);
-    if (type == G_VARIANT_TYPE_TUPLE) {
-        IBusSerializable *object;
-        object = ibus_serializable_deserialize (variant);
-        value = g_slice_new0 (GValue);
-        g_value_init (value, G_OBJECT_TYPE (object));
-        g_value_take_object (value, object);
-        return value;
-    }
-
-    typedef gchar *gstring;
-#define IF_ENTRY(TYPE, _type, signature)                    \
-    if (type ==  G_VARIANT_TYPE_##TYPE) {                   \
-        g##_type v;                                         \
-        g_variant_get (variant, signature, &v);             \
-        value = g_slice_new0 (GValue);                      \
-        g_value_init (value, G_TYPE_##TYPE);                \
-        g_value_set_##_type (value, v);                     \
-        return value;                                       \
-    }
-#define G_VARIANT_TYPE_CHAR G_VARIANT_TYPE_BYTE
-    IF_ENTRY(CHAR, char, "y");
-#undef G_VARIANT_TYPE_CHAR
-    IF_ENTRY(BOOLEAN, boolean, "b");
-#define G_VARIANT_TYPE_INT G_VARIANT_TYPE_INT32
-#define G_VARIANT_TYPE_UINT G_VARIANT_TYPE_UINT32
-    IF_ENTRY(INT, int, "i");
-    IF_ENTRY(UINT, uint, "u");
-#undef G_VARIANT_TYPE_INT
-#undef G_VARIANT_TYPE_UINT
-    IF_ENTRY(INT64, int64, "x");
-    IF_ENTRY(UINT64, uint64, "t");
-    IF_ENTRY(DOUBLE, double, "d");
-    IF_ENTRY(STRING, string, "s");
-
-    g_return_val_if_reached (NULL);
-}
-
 static void
 _serialize_cb (GQuark           key,
-               GValue          *value,
+               GVariant        *value,
                GVariantBuilder *array)
 {
     g_variant_builder_add (array, "{sv}",
-                g_quark_to_string (key), _g_value_serialize (value));
+                g_quark_to_string (key), g_variant_new_variant (value));
 }
 
 static gboolean
@@ -257,21 +156,26 @@ ibus_serializable_real_deserialize (IBusSerializable *object,
     GVariantIter *iter = NULL;
     g_variant_get_child (variant, 1, "a{sv}", &iter);
     while (g_variant_iter_loop (iter, "{&sv}", &key, &value)) {
-        ibus_serializable_set_attachment (object, key, _g_value_deserialize (value));
+        GVariant *attachment = g_variant_get_variant (value);
+        ibus_serializable_set_attachment (object,
+                                          key,
+                                          attachment);
+        g_variant_unref (attachment);
+        g_variant_unref (value);
     }
     g_variant_iter_free (iter);
     return 2;
 }
 
 static void
-_copy_cb (GQuark   key,
-          GValue  *value,
-          GData  **datalist)
+_copy_cb (GQuark     key,
+          GVariant  *value,
+          GData    **datalist)
 {
     g_datalist_id_set_data_full (datalist,
                                  key,
-                                 ibus_g_value_dup (value),
-                                 (GDestroyNotify) ibus_g_value_free);
+                                 g_variant_ref (value),
+                                 (GDestroyNotify) g_variant_unref);
 }
 
 static gboolean
@@ -289,50 +193,21 @@ ibus_serializable_real_copy (IBusSerializable *dest,
     return TRUE;
 }
 
-gboolean
-ibus_serializable_set_qattachment (IBusSerializable *object,
+void
+ibus_serializable_set_qattachment (IBusSerializable *serializable,
                                    GQuark            key,
-                                   const GValue     *value)
+                                   GVariant         *value)
 {
-    g_return_val_if_fail (IBUS_IS_SERIALIZABLE (object), FALSE);
-    g_return_val_if_fail (key != 0, FALSE);
-    g_return_val_if_fail (G_IS_VALUE (value), FALSE);
+    g_return_if_fail (IBUS_IS_SERIALIZABLE (serializable));
+    g_return_if_fail (key != 0);
 
-    IBusSerializablePrivate *priv;
-    priv = IBUS_SERIALIZABLE_GET_PRIVATE (object);
-
-    GType type = G_VALUE_TYPE (value);
-
-    switch (type) {
-    case G_TYPE_CHAR:
-    case G_TYPE_INT:
-    case G_TYPE_INT64:
-    case G_TYPE_UINT:
-    case G_TYPE_UINT64:
-    case G_TYPE_BOOLEAN:
-    case G_TYPE_DOUBLE:
-    case G_TYPE_FLOAT:
-    case G_TYPE_STRING:
-        g_datalist_id_set_data_full (&priv->attachments,
-                                      key,
-                                      ibus_g_value_dup (value),
-                                      (GDestroyNotify) ibus_g_value_free);
-        return TRUE;
-    }
-
-    if (g_type_is_a (type, IBUS_TYPE_SERIALIZABLE)) {
-        g_datalist_id_set_data_full (&priv->attachments,
-                                      key,
-                                      ibus_g_value_dup (value),
-                                      (GDestroyNotify) ibus_g_value_free);
-        return TRUE;
-    }
-
-    g_warning ("The value of %s is not support serializing", g_type_name (type));
-    return FALSE;
+    g_datalist_id_set_data_full (&serializable->priv->attachments,
+                                 key,
+                                 value ? g_variant_ref_sink (value) : NULL,
+                                 (GDestroyNotify) g_variant_unref);
 }
 
-const GValue *
+GVariant *
 ibus_serializable_get_qattachment (IBusSerializable *serializable,
                                    GQuark            key)
 {
@@ -340,7 +215,8 @@ ibus_serializable_get_qattachment (IBusSerializable *serializable,
     g_return_val_if_fail (IBUS_IS_SERIALIZABLE (serializable), NULL);
     g_return_val_if_fail (key != 0, NULL);
 
-    return (const GValue *) g_datalist_id_get_data (&serializable->priv->attachments, key);
+    return (GVariant *) g_datalist_id_get_data (
+            &serializable->priv->attachments, key);
 }
 
 void
@@ -351,7 +227,7 @@ ibus_serializable_remove_qattachment (IBusSerializable *serializable,
     g_return_if_fail (IBUS_IS_SERIALIZABLE (serializable));
     g_return_if_fail (key != 0);
 
-    g_datalist_id_remove_no_notify (&serializable->priv->attachments, key);
+    g_datalist_id_set_data (&serializable->priv->attachments, key, NULL);
 }
 
 IBusSerializable *
